#include <linux/kernel.h>
#include <linux/notifier.h>
#include <linux/device.h>
#include <linux/dca.h>

MODULE_LICENSE("GPL");

/* For now I'm assuming a single, global, DCA provider for the system. */

static spinlock_t dca_lock;

struct dca_provider *global_dca = NULL;

int dca_add_requester(struct device *dev)
{
	int err, slot;

	if (!global_dca)
		return -ENODEV;

	spin_lock(&dca_lock);
	slot = global_dca->ops->add_requester(global_dca, dev);
	spin_unlock(&dca_lock);
	if (slot < 0)
		return slot;

	err = dca_sysfs_add_req(global_dca, dev, slot);
	if (err) {
		spin_lock(&dca_lock);
		global_dca->ops->remove_requester(global_dca, dev);
		spin_unlock(&dca_lock);
		return err;
	}

	return 0;
}

int dca_remove_requester(struct device *dev)
{
	int slot;
	if (!global_dca)
		return -ENODEV;

	spin_lock(&dca_lock);
	slot = global_dca->ops->remove_requester(global_dca, dev);
	spin_unlock(&dca_lock);
	if (slot < 0)
		return slot;

	dca_sysfs_remove_req(global_dca, slot);
	return 0;
}

u8 dca_get_tag(int cpu)
{
	if (!global_dca)
		return -ENODEV;
	return global_dca->ops->get_tag(global_dca, cpu);
}

EXPORT_SYMBOL(dca_add_requester);
EXPORT_SYMBOL(dca_remove_requester);
EXPORT_SYMBOL(dca_get_tag);

struct dca_provider *alloc_dca_provider(struct dca_ops *ops, int priv_size)
{
	struct dca_provider *dca;
	int alloc_size;

	alloc_size = (sizeof(*dca) + priv_size);
	dca = kzalloc(alloc_size, GFP_KERNEL);
	if (!dca)
		return NULL;
	dca->ops = ops;

	return dca;
}

void free_dca_provider(struct dca_provider *dca)
{
	kfree(dca);
}

static BLOCKING_NOTIFIER_HEAD(dca_provider_chain);

int register_dca_provider(struct dca_provider *dca, struct device *dev)
{
	int err;

	if (global_dca)
		return -EEXIST;
	err = dca_sysfs_add_provider(dca, dev);
	if (err)
		return err;
	global_dca = dca;
	dca_debug_populate_all_tags();
	blocking_notifier_call_chain(&dca_provider_chain, DCA_PROVIDER_ADD, NULL);
	return 0;
}

void unregister_dca_provider(struct dca_provider *dca)
{
	if (!global_dca)
		return;
	blocking_notifier_call_chain(&dca_provider_chain, DCA_PROVIDER_REMOVE, NULL);
	dca_debug_remove_all_tags();
	global_dca = NULL;
	dca_sysfs_remove_provider(dca);
}

EXPORT_SYMBOL(alloc_dca_provider);
EXPORT_SYMBOL(free_dca_provider);
EXPORT_SYMBOL(register_dca_provider);
EXPORT_SYMBOL(unregister_dca_provider);

void dca_register_notify(struct notifier_block *nb)
{
	blocking_notifier_chain_register(&dca_provider_chain, nb);
}

void dca_unregister_notify(struct notifier_block *nb)
{
	blocking_notifier_chain_unregister(&dca_provider_chain, nb);
}

EXPORT_SYMBOL(dca_register_notify);
EXPORT_SYMBOL(dca_unregister_notify);

static int __init dca_init(void)
{
	int err;

	spin_lock_init(&dca_lock);

	err = dca_sysfs_init();
	if (err)
		return err;
	err = dca_debug_init();
	if (err) {
		dca_sysfs_exit();
		return err;
	}
	return 0;
}

static void __exit dca_exit(void)
{
	dca_debug_exit();
	dca_sysfs_exit();
}

module_init(dca_init);
module_exit(dca_exit);

